@@ -84,6 +84,10 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
         lavh1 <- lavobject@h1
     }
 
+    if(lavdata@data.type == "none") {
+        lavsamplestats <- NULL
+    }
+
     # conditional.x ? check res.cov[[1]] slot
     conditional.x <- FALSE
     if(!is.null(lavsamplestats) && !is.null(lavsamplestats@res.cov[[1]])) {
@@ -155,6 +159,13 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
     categorical   <- any(ov$type == "ordered")
     ngroups       <- lavdata@ngroups
     nlevels       <- lavdata@nlevels
+    if(lavoptions$estimator == "catML") {
+        categorical <- FALSE
+    }
+    correlation <- FALSE
+    if(!is.null(lavoptions$correlation)) {
+        correlation  <- lavoptions$correlation
+    }
 
     # what with fixed.x?
     # - does not really matter; fit will be saturated anyway
@@ -269,11 +280,17 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
             block <- c(block, rep(b,  nvar))
             group <- c(group, rep(g,  nvar))
             level <- c(level, rep(l,  nvar))
-            free  <- c(free,  rep(1L, nvar))
+            if(correlation) {
+                free <- c(free,  rep(0L, nvar))
+            } else {
+                free <- c(free,  rep(1L, nvar))
+            }
             exo   <- c(exo,   rep(0L, nvar))
 
             # starting values -- variances
-            if(!is.null(sample.cov)) {
+            if(correlation) {
+                ustart <- c(ustart, rep(1, nvar))
+            } else if(!is.null(sample.cov)) {
                 ustart <- c(ustart, diag(sample.cov))
             } else {
                 ustart <- c(ustart, rep(as.numeric(NA), nvar))
@@ -369,6 +386,19 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
                 }
             } # categorical
 
+            # correlation structure?
+            if(!categorical && correlation) {
+                nel <- nvar
+                lhs   <- c(lhs, ov.names)
+                 op   <- c(op, rep("~*~", nel))
+                rhs   <- c(rhs, ov.names)
+                block <- c(block, rep(b,  nel))
+                group <- c(group, rep(g,  nel))
+                level <- c(level, rep(l,  nel))
+                free  <- c(free,  rep(0L, nel))
+                exo   <- c(exo,   rep(0L, nel))
+               ustart <- c(ustart, rep(1, nel))
+            }
 
             # meanstructure?
             if(meanstructure) {
@@ -387,7 +417,9 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
                 # if multilevel, level=1 has fixed zeroes
                 if(nlevels > 1L && l == 1L) {
                     WITHIN <- rep(0L, nel)
-                    WITHIN[Lp$within.idx[[2]]] <- 1L
+                    # FIXME: assuming 1 group
+                    within.idx <- match(Lp$within.idx[[2]], Lp$ov.idx[[1]])
+                    WITHIN[within.idx] <- 1L
                     free  <- c(free, WITHIN)
                 } else {
                     free  <- c(free,  rep(1L, nel))
@@ -682,9 +714,15 @@ lav_partable_unrestricted_chol <- function(lavobject = NULL,
             # block
             b <- b + 1L
 
-            ov.names     <- lavpta$vnames$ov[[b]]
-            ov.names.x   <- lavpta$vnames$ov.x[[b]]
-            ov.names.nox <- lavpta$vnames$ov.nox[[b]]
+            if(is.null(lavpta)) {
+                ov.names     <- lavdata@ov.names[[b]]
+                ov.names.x   <- lavdata@ov.names.x[[b]]
+                ov.names.nox <- ov.names[!ov.names %in% ov.names.x]
+            } else {
+                ov.names     <- lavpta$vnames$ov[[b]]
+                ov.names.x   <- lavpta$vnames$ov.x[[b]]
+                ov.names.nox <- lavpta$vnames$ov.nox[[b]]
+            }
 
             # only for multilevel, overwrite sample.cov and sample.mean
             if(nlevels > 1L) {
@@ -735,18 +773,20 @@ lav_partable_unrestricted_chol <- function(lavobject = NULL,
             lower <- c(lower, rep(0.0, nvar)) # lower bound!
 
             # d) LOADINGS other
-            tmp <- utils::combn(ov.names, 2)
-            pstar <- ncol(tmp)
-            lhs <- c(lhs, paste("f", tmp[1,], sep = ""))
-             op <- c(op,   rep("=~", pstar))
-            rhs <- c(rhs, tmp[2,])
-            block <- c(block, rep(b,   pstar))
-            group <- c(group, rep(g,   pstar))
-            level <- c(level, rep(l,   pstar))
-            free  <- c(free,  rep(1L,  pstar))
-            exo   <- c(exo,   rep(0L,  pstar))
-            lower <- c(lower, rep(-Inf, pstar))
-            ustart<- c(ustart,rep(as.numeric(NA), pstar))
+            if(length(ov.names) > 1L) {
+                tmp <- utils::combn(ov.names, 2)
+                pstar <- ncol(tmp)
+                lhs <- c(lhs, paste("f", tmp[1,], sep = ""))
+                 op <- c(op,   rep("=~", pstar))
+                rhs <- c(rhs, tmp[2,])
+                block <- c(block, rep(b,   pstar))
+                group <- c(group, rep(g,   pstar))
+                level <- c(level, rep(l,   pstar))
+                free  <- c(free,  rep(1L,  pstar))
+                exo   <- c(exo,   rep(0L,  pstar))
+                lower <- c(lower, rep(-Inf, pstar))
+                ustart<- c(ustart,rep(as.numeric(NA), pstar))
+            }
 
             # meanstructure?
             if(meanstructure) {
@@ -764,7 +804,8 @@ lav_partable_unrestricted_chol <- function(lavobject = NULL,
                 # if multilevel, level=1 has fixed zeroes
                 if(nlevels > 1L && l == 1L) {
                     WITHIN <- rep(0L, nel)
-                    WITHIN[Lp$within.idx[[2]]] <- 1L
+                    within.idx <- match(Lp$within.idx[[2]], Lp$ov.idx[[1]])
+                    WITHIN[within.idx] <- 1L
                     free  <- c(free, WITHIN)
                 } else {
                     free  <- c(free,  rep(1L, nel))
